/*
#### Copyright (c) 2001 TurboLinux All Rights Reserved
####              Distributable under the terms of the GPL.
#### File:        usbdet.c
#### Author:      Go Taniguchi <go@turbolinux.co.jp>
#### Description: usb uhci ohci detection
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>

int print = 0;
char pci_path[] = "/proc/bus/pci";
int detusb = 0;

int det_usb(char *path)
{
	int fd;
	int ret = 0;
	unsigned char aaa[16];
	fd = open(path, O_RDONLY);
	if (fd == -1) return 0;
	if(read(fd, aaa, 12) < 12) return 0;
	close(fd);
	if(aaa[10] == 3 && aaa[11] == 0x0c){
		if(print)
			printf("%02X%02X:%02X%02X\t", aaa[1], aaa[0], aaa[3], aaa[2]);
		if(aaa[9] == 0){
			ret = 1;
			if(print) printf("UHCI");
		}else{
			if(print) printf("OHCI");
			ret = 2;
		}
		if(print) printf(" USB Controller Detected\n");
	}
	return ret;
}

void find_dir(char *path, int type)
{
	DIR *dir;
	struct dirent *dirp;
	int len, ret = 0;
	char *newpath;
	struct stat sbuf;

	dir = opendir(path);
	while((dirp = readdir(dir)) != NULL){
		if(dirp->d_ino == 0) break;
		if(/*strcmp(dirp->d_name, "devices") == 0 || */
		   strcmp(dirp->d_name, ".") == 0 ||
		   strcmp(dirp->d_name, "..") == 0)
			continue;
		len = strlen(path) + strlen(dirp->d_name) + 2;
		newpath = (char *)malloc(len);
		strcpy(newpath, path);
		strcat(newpath, "/");
		strcat(newpath, dirp->d_name);
		stat(newpath, &sbuf);
		if(type == 0 && S_ISDIR(sbuf.st_mode) != 0){
			find_dir(newpath, 1);
			free(newpath);
		}else{
			ret = det_usb(newpath);
			free(newpath);
			if(ret == 1 || ret == 2){
				detusb = ret;
				if(print == 0) break;
			}
		}
	}
	closedir(dir);
}

int main(int argc, char **argv)
{
	DIR *dir;
	struct dirent *dirp;
	if(argc > 1 && strncmp(argv[1], "--print", sizeof("--print")) == 0)
		print = 1;
	if (access(pci_path, R_OK) != 0){
		printf("Can not access %s\n", pci_path);
		return -1;
	}
	find_dir(pci_path, 0);
	return detusb;
}
